!**************************************************************************************************
! Establishment of woody PFTs
!**************************************************************************************************
SUBROUTINE establish &
    (year, counter, fire_clo, tmp_coldest_20yr_ave, gdd_20yr_ave, &
    par, par_grass, par_grass_RunningRecord, stat_water_RunningRecord, &
    pool_litter, c_uptake, pft_exist, patch_vacant, &
    pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
    mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
    bole_x, bole_y, crown_x, crown_y, &
    tree_exist, height_limit, radius_limit, mort_regu1, mort_regu2, &
    stat_leaf, stat_leaf_RunningRecord, stat_leaf_crowntop, stat_leaf_crownbottom)

USE data_structure
implicit none

!***************************   Define Establish Conditions   *************************
   !  -- Explanation for each variables (All variables are PFT specific) --
   !
   ! est_capacity    -> Logical value (true or false).
   !                    It indicates wheather a PFT can establish or not on this site.
   ! est_possibility -> Frequency that a establishment patch is allocated to a PFT (non dimension).    
   !                    Thus, sum of est_possibility for all PFTs is always 1.0
   ! P_establish     -> Actual establishment rate when a patch was allocated to a PFT
   
   !Define scenario for establishment (0 or 1 or 2 or 3)
   integer,parameter::scenario = 3
   !   0 -> no trees can establish
   !   1 -> only specified woody PFTs can establish
   !   2 -> every potentially eatablishable PFT have same chance of establishment
   !   3 -> in proportion of existing biomass after specific year
   
   !Scenario 3: lenght of year until establishment pattern will change
   integer,parameter::year_change= 50
   
   !Scenario 1: establishment switch  (0 or 1)
   logical,dimension(PFT_no),parameter:: &
   est_switch=(/.false.,.false.,.false.,.false.,.false.,.true.,.false.,.false.,.false.,.false./)
   !            1.TrBE  2.TrBR  3.TeNE  4.TeBE  5.TeBS  6.BoNE  7.BoNS 8.BoBS    9.C3   10.C4

!***************************   Define Arguments   *************************
integer,intent(IN)::year                 !loop for year (1-Simulation_year)
integer,intent(IN)::counter              !simulation day (1-Simulation_year*365)
integer,intent(IN)::fire_clo             !counter number of last fire (yr)
real   ,intent(IN)::tmp_coldest_20yr_ave !20yr running mean of tmp_coldest_month (Celcius)
real   ,intent(IN)::gdd_20yr_ave         !20yr running mean of growing-degree-day-sum
real   ,intent(IN)::par                  !par over forest (micro mol photon m-2 s-1)
real   ,intent(IN)::par_grass            !par on grass layer (micro mol photon m-2 s-1)
real,dimension(365)       ,intent(IN)::par_grass_RunningRecord  !1yr record of par_grass
real,dimension(PFT_no,365),intent(IN)::stat_water_RunningRecord !1yr record of stat_water

real   ,intent(INOUT)::pool_litter       !litter pool in a stand (g / stand)
real   ,intent(INOUT)::c_uptake          !monthly carbon uptake (C g / month stand)

logical,dimension(PFT_no)         ,intent(IN)::pft_exist    !PFT exist flag
logical,dimension(1:Dived,1:Dived),intent(IN)::patch_vacant !safe site for establishment

integer,dimension(1:Max_no),intent(INOUT)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(1:Max_no),intent(INOUT)::age            !tree age (year)
integer,dimension(1:Max_no),intent(INOUT)::height         !tree height from 1.3m above ground(STEP)
integer,dimension(1:Max_no),intent(INOUT)::bole           !bole length (STEP)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_diameter !crown diameter (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_area     !crown area (m2)
real   ,dimension(1:Max_no),intent(INOUT)::la             !leaf area  (m2)
real   ,dimension(1:Max_no),intent(INOUT)::mass_leaf      !biomass of foliage  (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_trunk     !biomass of stem     (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_root      !biomass of root     (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_stock     !biomass for stock   (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_available !biomass available   (g)
real   ,dimension(1:Max_no),intent(INOUT)::bole_x         !x_location of bole  (m)
real   ,dimension(1:Max_no),intent(INOUT)::bole_y         !y_location of bole  (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_x        !x_location of crown (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_y        !y_location of crown (m)
logical,dimension(1:Max_no),intent(INOUT)::tree_exist     !tree exist or not (T or F)
integer,dimension(1:Max_no),intent(INOUT)::height_limit   !max_tree_height proximate trees permit
real   ,dimension(1:Max_no),intent(INOUT)::radius_limit   !max_canopy_radius proximate trees permit
real   ,dimension(1:Max_no),intent(INOUT)::mort_regu1     !NPP sum within the previous year (g dm)
real   ,dimension(1:Max_no),intent(INOUT)::mort_regu2     !mean leaf area of last year (m2/day)

real,dimension(1:Max_no    ),intent(INOUT)::stat_leaf               !(income-cost)/leaf_mass
real,dimension(1:Max_no,1:7),intent(INOUT)::stat_leaf_RunningRecord !running record of stat_leaf
real,dimension(1:Max_no    ),intent(INOUT)::stat_leaf_crowntop      !stat_leaf of crown top
real,dimension(1:Max_no,10 ),intent(INOUT)::stat_leaf_crownbottom   !stat_leaf of crown bottom

!Define local variables
real   ,dimension(PFT_no)::est_possibility !establishment possibility of each woody PFT (total=1.0)
logical,dimension(PFT_no)::est_capacity    !capacity of establishment for enac woody PFT
real   ,dimension(PFT_no)::mass_sum        !total biomass of examining tree (g)
integer                    no_possible     !number of woody PFTs that can establish in this grid
integer,dimension(1:PFT_no)::dry_month        !maximum successive number of dry month
real   ,dimension(1:12)    ::stat_water_month !monthly average of stat_water

integer new        !number of new sapling
integer i, j, k, p !for loop counter
integer no         !for loop of no
real    x          !for general usage

!*********************   Compute variables that control establishment    *******************
!When tree number is maximum, omit entire subroutine
   no = 0
   do i=1, Max_no
     if (tree_exist(i)) no=no+1
   end do
   if (no >= Max_no) return
   
!If fire occured within 10 years, omit entire subroutine
!   if ( counter-fire_clo <= 10*365 ) return
   
!Determine dry_month (maximum consecutive of drought month for each PFT)
   dry_month(1:PFT_no) = 0
   Do p=1, PFT_no
      stat_water_month(1:12) = 0.0
      
      !sum up stat_water for each month
      Do i=1, 365
         stat_water_month(Month(i)) = stat_water_month(Month(i)) + stat_water_RunningRecord(p,i)
      End do
      
      !make average of stat_water for each month
      Do i=1, 12
         stat_water_month(i) = stat_water_month(i) / real(Day_in_month(i))
      End do
      
      !determine maximum consecutive of drought month for each PFT
      j = 0
      Do i=1, 12*2
         j=j+1
         if (i<=12) then
            if ( stat_water_month(i)    > 0.3 ) j=0
         else
            if ( stat_water_month(i-12) > 0.3 ) j=0
         endif
         dry_month(p) = max( dry_month(p) , j )
      End do
      
      dry_month(p) = min(dry_month(p), 12)
   End do
   
!Determine which PFT can establish (climatic range of establishment)
   est_capacity(:) = .true.
   do p = 1, PFT_no
      x = sum( par_grass_RunningRecord(1:365) ) / 365.0
      if (Life_type(p)         .ne. 0          ) est_capacity(p)=.false.
      if (tmp_coldest_20yr_ave >=   TC_max(p)  ) est_capacity(p)=.false.
      if (gdd_20yr_ave         <=   GDD_min(p) ) est_capacity(p)=.false.
      if (dry_month(p)         >    DM_max(p)  ) est_capacity(p)=.false.
      if (x                    <=   PAR_min(p) ) est_capacity(p)=.false.
   end do

!Determine establishment probability (scenario specific)
   est_possibility(:) = 0.0
   no_possible        = 0  
   
   IF     (scenario == 0) then
      !*** SCENARIO 0 ***
      est_possibility(:) = 0.0
      no_possible        = 0  
      
   ELSEIF (scenario == 1) then
      !*** SCENARIO 1 ***
      do p = 1, PFT_no
      if ( Life_type(p)==0 .and. est_capacity(p) .and. est_switch(p) ) then
         no_possible        = no_possible + 1
         est_possibility(p) = 1.0
      endif
      enddo
      
      if (no_possible .ne. 0) then
         est_possibility(:) = est_possibility(:) / real(no_possible)
      end if
      
   ELSEIF (scenario== 3 .and. year>=year_change .and. counter-fire_clo>=50*365 ) then
      !*** SCENARIO 3 (after year_change) ***
      mass_sum(:) = 0.0 !sum of biomass for each woody PFT with establishment capacity
      do no = 1, Max_no
      if ( tree_exist(no) .and. est_capacity(pft(no)) ) then
         mass_sum(pft(no)) = mass_sum(pft(no)) + &
         mass_leaf(no)+mass_trunk(no)+mass_root(no)+mass_available(no)+mass_stock(no)
      end if
      end do
      
      if ( sum(mass_sum)==0.0 ) return
      
      do p = 1, PFT_no
      if ( Life_type(p)==0 .and. est_capacity(p) ) then
         est_possibility(p) = mass_sum(p) / sum(mass_sum)
         if (est_possibility(p) > 0.00) no_possible=no_possible+1
      endif
      end do
      
   ELSE
      !*** SCENARIO 2 and SENERIO 3 (until year_change) ***
      !determine woody PFTs that can potentially establish in this grid
      
      do p = 1, PFT_no
      if ( Life_type(p)==0 .and. est_capacity(p) ) then
         est_possibility(p) = 1.0
         no_possible        = no_possible + 1
      end if
      end do
      
      !give same establishment possibility for each favoured PFT 
      if (no_possible .ne. 0) then
         est_possibility(:) = est_possibility(:) / real(no_possible)
      end if
      
   ENDIF
   if (no_possible == 0) return

!Display establishment possibility for each woody PFT (for tuning)
!if (Logging==1) then
!    write(*,*)                 '  '
!    write(*,'(a55)')           '   ***** establishment possibility for each PFT *****'
!    write(*,'(a55)')           '   1       2       3       4       5       6       7       8'
!    write(*,'(a5, 8(f5.3,3X))')'   ', est_possibility(1:8)
!endif

!*********************   Establishment processes    *******************
!loop for each ground mesh
new = 1
Do i = 1, Dived
Do j = 1, Dived
   
   !K: determin PFT of newly establish tree
   x = randf(I_seed)
   k = 1
   do while ( sum(est_possibility(1:k)) < x .and. k<PFT_no)
      k = k+1
   end do
   
   !x: determine adjusted establishment rate of PFT k
   x = P_establish(k) * real(Max_loc**2) / real(Dived**2)
   
if ( patch_vacant(i,j) ) then
if ( x > randf(I_seed) ) then
   !determine new sapling ID-number
   do while ( tree_exist(new) )
      new = new + 1
   end do
   if ( new > Max_no ) return
   
   !initiate new sapling
   pft(new)        = k
   tree_exist(new) = .true.
   bole_x (new)    = real(i) * ( real(Max_loc) / real(Dived) ) - 0.5*real(Max_loc)/real(Dived)
   bole_y (new)    = real(j) * ( real(Max_loc) / real(Dived) ) - 0.5*real(Max_loc)/real(Dived)
   crown_x(new)    = bole_x (new)
   crown_y(new)    = bole_y (new)
   
   Call init_tree &
   (par, par_grass, pool_litter, c_uptake, &
   pft(new), age(new), height(new), bole(new), dbh_heartwood(new), dbh_sapwood(new), &
   crown_diameter(new), crown_area(new), la(new), &
   mass_leaf(new), mass_trunk(new), mass_root(new), mass_stock(new), mass_available(new), &
   tree_exist(new), height_limit(new), radius_limit(new), &
   mort_regu1(new), mort_regu2(new), &
   stat_leaf(new), stat_leaf_RunningRecord(new,1:7), &
   stat_leaf_crowntop(new), stat_leaf_crownbottom(new,1:10) )
   
end if
end if
End do
End do

END SUBROUTINE establish



!**************************************************************************************************
! Change priority grass
!**************************************************************************************************
SUBROUTINE grass_priority &
     (type_grass, C3g_no, C4g_no, &
     phenology, pheno_clo, pheno_cls, pool_litter, npp_annual, &
     gmass_leaf, gmass_root, gmass_available, gmass_stock)

USE data_structure
implicit none

!*************************   Define augments   *************************
integer,intent(INOUT)::type_grass !grass type priority (3:C3, 4:C4)
integer,intent(INOUT)::C3g_no      !PFT number of C3 grass
integer,intent(INOUT)::C4g_no      !PFT number of C4 grass

logical,dimension(PFT_no),intent(INOUT)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(PFT_no),intent(INOUT)::pheno_clo !counter number of last leaf onset
integer,dimension(PFT_no),intent(INOUT)::pheno_cls !counter number of last leaf shedding

real                  ,intent(IN)::pool_litter    !litter pool in a stand (g dm/stand)
real,dimension(PFT_no),intent(IN)::npp_annual     !NPP for each pft (g dm/stand/year)

real,dimension(2) ,intent(INOUT)::gmass_leaf      !c3/c4 grass foliage   biomass (g/m2)
real,dimension(2) ,intent(INOUT)::gmass_root      !c3/c4 grass root      biomass (g/m2)
real,dimension(2) ,intent(INOUT)::gmass_available !c3/c4 grass available biomass (g/m2)
real,dimension(2) ,intent(INOUT)::gmass_stock     !c3/c4 grass stock     biomass (g/m2)

!*************************   Main procedure   *************************
IF     (type_grass==4 .and. npp_annual(C3g_no)*9.0>npp_annual(C4g_no) ) then
   !C4 -> C3 change
   if (Logging==1) then
      write (*,*) 'priority grass changed: C4 -> C3', npp_annual(C3g_no)*9.0, npp_annual(C4g_no)
      write (*,*)
   endif
   
   type_grass  = 3
   call exchange_r (gmass_leaf(1)     , gmass_leaf(2)      )
   call exchange_r (gmass_root(1)     , gmass_root(2)      )
   call exchange_r (gmass_available(1), gmass_available(2) )
   call exchange_r (gmass_stock(1)    , gmass_stock(2)     )
   call exchange_l (phenology(C3g_no) , phenology(C4g_no)  )
   call exchange_i (pheno_clo(C3g_no) , pheno_clo(C4g_no)  )
   call exchange_i (pheno_cls(C3g_no) , pheno_cls(C4g_no)  )
   
ELSEIF (type_grass==3 .and. npp_annual(C3g_no)<npp_annual(C4g_no)*9.0 ) then
   !C3 -> C4 change
   if (Logging==1) then
      write (*,*) 'priority grass changed: C3 -> C4', npp_annual(C3g_no), npp_annual(C4g_no)*9.0
      write (*,*)
   endif
   
   type_grass  = 4
   call exchange_r (gmass_leaf(1)     , gmass_leaf(2)      )
   call exchange_r (gmass_root(1)     , gmass_root(2)      )
   call exchange_r (gmass_available(1), gmass_available(2) )
   call exchange_r (gmass_stock(1)    , gmass_stock(2)     )
   call exchange_l (phenology(C3g_no) , phenology(C4g_no)  )
   call exchange_i (pheno_clo(C3g_no) , pheno_clo(C4g_no)  )
   call exchange_i (pheno_cls(C3g_no) , pheno_cls(C4g_no)  )
   
END IF

END SUBROUTINE grass_priority



!**************************************************************************************************
! Calculate moisture factor for fire regime routine
!**************************************************************************************************
Subroutine moist_factor_calc &
         (moist_factor, pft, &
         mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, &
         gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass, C3g_no, C4g_no)

USE data_structure
implicit none

!*************************   Define augments   *************************
real,intent(OUT)::moist_factor   !adjusted moisture factor for fire spread

integer,dimension(1:Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
real   ,dimension(1:Max_no),intent(IN)::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no),intent(IN)::mass_trunk     !biomass of stem    (g)
real   ,dimension(1:Max_no),intent(IN)::mass_root      !biomass of root    (g)
real   ,dimension(1:Max_no),intent(IN)::mass_stock     !biomass for stock  (g)
real   ,dimension(1:Max_no),intent(IN)::mass_available !biomass available  (g)
logical,dimension(1:Max_no),intent(IN)::tree_exist     !tree exist or not (.TRUE. or .FALSE.) 

real,dimension(1:2),intent(IN)::gmass_leaf      !grass foliage    biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_root      !grass root       biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_available !grass available  biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_stock     !herbaceous stock biomass (g/m2)
integer            ,intent(IN)::type_grass      !grass type (3:C3, 4:C4)
integer            ,intent(IN)::C3g_no          !PFT number of C3 grass
integer            ,intent(IN)::C4g_no          !PFT number of C4 grass

!*************************   Define local variables   *************************
real,dimension(PFT_no)::mass_sum     !total aboveground biomass for each PFT (g/stand)
real                    mass_overall !total aboveground biomass in this stand (g/stand)
integer                 no, i        !loop counter
real                    a1, a2

!*************************   Main procedures   *************************
!Reset variables
mass_sum(:)  = 0.0
mass_overall = 0.0
moist_factor = 0.0

!mass_total(PFT_no): total above ground biomass for each PFT
   !woody PFTs
   Do no = 1, Max_no
   if (tree_exist(no)) then
      mass_sum(pft(no)) = mass_sum(pft(no)) + mass_leaf(no) + mass_trunk(no) + mass_available(no)
   end if
   End do
    
   !herbaceous PFTs
    a1 = gmass_leaf(1) + gmass_available(1) !above ground biomass of C3 glass for unit area
    a2 = gmass_leaf(2) + gmass_available(2) !above ground biomass of C4 glass for unit area
    
   if (type_grass==3) then
      mass_sum(C3g_no) = a1 * real(Max_loc)*real(Max_loc) * 0.9
      mass_sum(C4g_no) = a2 * real(Max_loc)*real(Max_loc) * 0.1
   else
      mass_sum(C3g_no) = a1 * real(Max_loc)*real(Max_loc) * 0.1
      mass_sum(C4g_no) = a2 * real(Max_loc)*real(Max_loc) * 0.9
   endif

!mass_overall: total biomass in this grid
   mass_overall = sum(mass_sum) !grand total biomass in this forest stand
   mass_overall = max(0.01, mass_overall)

!moistfactor: adjusted litter moisture weighting factor
   Do i = 1, PFT_no
      moist_factor = moist_factor + Moisture_extinction(i) * (mass_sum(i) / mass_overall)
   End do
   
!adjust to the upper limit and lower limit of the value
   moist_factor = min(1.0, max(0.01, moist_factor))

End subroutine moist_factor_calc



!**************************************************************************************************
! Fire regime (for each year calculation)
!**************************************************************************************************
Subroutine fire_regime &
   ( fire_factor, fire_clo, pool_litter, c_emission, phenology, &
   pft, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
   lai_grass, type_grass, year, counter )

USE data_structure
implicit none

!Define augments
real   ,intent(IN)   ::fire_factor  !factor for fire probability calculation
integer,intent(INOUT)::fire_clo     !counter number of last fire
real   ,intent(INOUT)::pool_litter  !litter pool in a stand (g / stand )
real   ,intent(INOUT)::c_emission   !monthly carbon emission (C g / month stand)

logical,dimension(PFT_no),intent(INOUT)::phenology !Phenological status (T:foliation, F:dormance)

integer,dimension(1:Max_no),intent(INOUT)::pft            !Plant Functional Type (1-PFT_no)
real   ,dimension(1:Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_trunk     !biomass of stem    (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_root      !biomass of root    (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_stock     !biomass for stock  (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_available !biomass available  (g)
logical,dimension(1:Max_no),intent(INOUT)::tree_exist     !tree exist or not (.TRUE. or .FALSE.) 

integer,dimension(1:Max_no),intent(INOUT)::age            !Tree age (year)
integer,dimension(1:Max_no),intent(INOUT)::height         !height from 1.3m above ground (STEP)
integer,dimension(1:Max_no),intent(INOUT)::bole           !Bole length (STEP)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_heartwood  !Hertwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_sapwood    !Sapwood diameter at  1.3m   (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_diameter !Crown diameter   (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_area     !Crown area      (m2)
real   ,dimension(1:Max_no),intent(INOUT)::la             !Leaf area      (m2)

integer            ,intent(INOUT)::type_grass      !priority grass type (3:C3, 4:C4)
real               ,intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
real,dimension(1:2),intent(INOUT)::gmass_leaf      !grass foliage biomass (g/m2)
real,dimension(1:2),intent(INOUT)::gmass_root      !grass root biomass (g/m2)
real,dimension(1:2),intent(INOUT)::gmass_available !grass available biomass (g/m2)
real,dimension(1:2),intent(INOUT)::gmass_stock     !grass stock biomass (g/m2)

integer,intent(IN):: year    !loop counter for year (1-Simulation_year)
integer,intent(IN):: counter !simulation day (1-Simulation_year*365)

!Define local variables
real    Min_gmass_stock !minimum stock mass of grass layer (g DM/m2)
integer no              !loop counter for individual trees
integer i               !loop counter
real    fire_prob       !probability of fire occurance in a year
real    total_gmass_ag  !total aboveground grass biomass in this stand (g dm / stand)
real    x               !for general usage

!Set parameter
   Min_gmass_stock = 50.0 !gDM / m2
   
!Fire_prob: calculate fire probability [A(s) in eq.37]
   x         = fire_factor - 1.0
   fire_prob = fire_factor * EXP( x / (0.45*x*x*x + 2.83*x*x + 2.96*x + 1.04) )
   
!Examine this site satisfies minimum-fuel-load
   
    !total_gmass_ag: total grass foliage biomass in the forest (g dm / forest)
    if (type_grass==3) then
       total_gmass_ag = ( 0.9*gmass_leaf(1) + 0.1*gmass_leaf(2) ) * Max_loc*Max_loc
    else
       total_gmass_ag = ( 0.1*gmass_leaf(1) + 0.9*gmass_leaf(2) ) * Max_loc*Max_loc
    end if
   
   !x: total fuel load in this forest stand (g C / m2)
    x = sum(mass_leaf)+sum(mass_trunk)+sum(mass_available) + total_gmass_ag + pool_litter
    x = x * C_in_drymass / (Max_loc * Max_loc)
    
    !effect of threshold of fuel load on fire
   if ( Fuel_min > x  ) fire_prob = 0.0

!TMP (for succession experiment1)
!if ( year => 1001 )          then !after 1000yr
!    if ( mod(year, 200)==1 ) then !for each 200yr interval
!       fire_prob = 1.0
!    else
!       fire_prob = 0.0
!    endif
!else
!    fire_prob = 0.0
!endif

!TMP (for succession experiment2)
!if ( mod(year,1000)==1 ) then
!   fire_prob = 1.0
!end if

!When fire occurs
   If ( (fire_prob>randf(I_seed)) .and. (counter-fire_clo>365) ) then
      if (Logging==1) write(*,*) 'Fire occured!, ', counter
      fire_clo = counter
      
      !Litter pool -> 50% lost
      c_emission   = c_emission + 0.50 * pool_litter * C_in_drymass
      pool_litter  = 0.50 * pool_litter
      
      !Grass PFTs -> total above ground biomass lost
      c_emission = c_emission + C_in_drymass * total_gmass_ag
      gmass_leaf (1) = 0.0
      gmass_leaf (2) = 0.0
      
      if (type_grass==3) then
         x = 0.9 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(1) ) + &
             0.1 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(2) )    
      else
         x = 0.1 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(1) ) + &
             0.9 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(2) )    
      end if
      pool_litter = max(0.0, pool_litter - x)
      
      gmass_stock(1) = max( gmass_stock(1), Min_gmass_stock )
      gmass_stock(2) = max( gmass_stock(2), Min_gmass_stock )
      
      !Woody PFT: A fraction of trees were killed, and their above ground biomass lost
      Do no = 1, Max_no
      if (tree_exist(no)) then
      if ( randf(I_seed) >  M3(pft(no)) ) then
         !for dead plants
         c_emission   = c_emission                           + &
                        C_in_drymass * mass_leaf(no)         + &
                        C_in_drymass * mass_trunk(no) * 0.50    
         
         pool_litter = pool_litter                           + &
                       mass_trunk(no) * 0.50                 + &
                       mass_root(no)                         + &
                       mass_stock(no)                        + &
                       mass_available(no)                       
         
         tree_exist(no)      = .false.
         pft(no)            = 0
         age(no)            = 0
         height(no)         = 0
         bole(no)           = 0
         dbh_sapwood(no)    = 0.0
         dbh_heartwood(no)  = 0.0
         crown_area(no)     = 0.0
         crown_diameter(no) = 0.0
         la(no)             = 0.0
         mass_leaf(no)      = 0.0
         mass_root(no)      = 0.0
         mass_trunk(no)     = 0.0
         mass_stock(no)     = 0.0
         mass_available(no) = 0.0
      else
         !for survived trees
         c_emission    = c_emission + C_in_drymass * mass_leaf(no)
         mass_leaf(no) = 0.0
         la(no)        = 0.0
      end if
      end if
      End do
      
        !Deciduous PFTs -> change to dormance phase
      Do i=1, PFT_no
      if ( Phenology_type(i) .ne. 0.0 ) then 
         phenology(i) = .false.
      End if
      End do
      
   Endif
   
End subroutine fire_regime



!**************************************************************************************************
! mortality of individual 'no' for the current year
!**************************************************************************************************
SUBROUTINE mortality &
      (tree_exist, pft, age, height, bole, dbh_heartwood, dbh_sapwood, &
      crown_diameter, crown_area, la, &
      mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
      height_limit, radius_limit, &
      stat_leaf, stat_leaf_RunningRecord, stat_leaf_crowntop, stat_leaf_crownbottom, &
      mort_regu1, mort_regu2, mort_regu3, &
      pool_litter, tmp_coldest_20yr_ave, tmp_hottest_20yr_ave)

USE data_structure
implicit none

!Define arguments
logical,dimension(1:Max_no),intent(INOUT)::tree_exist     !tree exist or not
integer,dimension(1:Max_no),intent(INOUT)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(1:Max_no),intent(INOUT)::age            !Tree age (year)
integer,dimension(1:Max_no),intent(INOUT)::height         !Tree height from 1.3m above ground(STEP)
integer,dimension(1:Max_no),intent(INOUT)::bole           !Bole length (STEP)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_heartwood  !Hertwood diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_sapwood    !Sapwood diameter at  1.3m (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_diameter !Crown diameter   (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_area     !Crown area      (m2)
real   ,dimension(1:Max_no),intent(INOUT)::la             !Leaf area       (m2)
real   ,dimension(1:Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_trunk     !biomass of stem    (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_root      !biomass of root    (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_stock     !biomass for stock  (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_available !biomass available  (g)

integer,dimension(1:Max_no),intent(INOUT)::height_limit !max tree height proximate trees permit
real   ,dimension(1:Max_no),intent(INOUT)::radius_limit !max canopy radius proximate trees permit

real,dimension(1:Max_no    ),intent(INOUT)::stat_leaf           !(income-expenditure)/leaf_mass
real,dimension(1:Max_no,1:7),intent(INOUT)::stat_leaf_RunningRecord !running record of stat_leaf
real,dimension(1:Max_no    ),intent(INOUT)::stat_leaf_crowntop    !stat_leaf @ the top crown disk
real,dimension(1:Max_no,10 ),intent(INOUT)::stat_leaf_crownbottom !stat_leaf @ bottom crown disks

real,dimension(1:Max_no),intent(INOUT)::mort_regu1 ! NPP annual (g dm / individual)
real,dimension(1:Max_no),intent(INOUT)::mort_regu2 ! mean leaf area of last year (m2/day)

real,intent(INOUT)::pool_litter          !litter pool in a stand (g / stand )

real,intent(IN)::tmp_coldest_20yr_ave !20yr running mean of tmp_coldest_month (Celcius)
real,intent(IN)::tmp_hottest_20yr_ave !20yr running mean of tmp_hottest_month (Celcius)
real,intent(IN)::mort_regu3           ! heat stress for each year

!Define local variable
integer no           !loop counter for individual
integer p            !PFT number

!real    npp_year     !individual NPP for the last one year
real    greff        !growth efficiency
real    mort_greff   !mortality as a result of low growth efficiency
real    mort_heart   !mortality as a result of heart stress
real    mort_lim     !mortality as a result of bioclimic limits
real    mort_etc     !mortality as a result of other factors
real    mort_total   !mortality as a result of above all factors

!write(*,*) 'tmp_hottest_20yr_ave-tmp_coldest_20yr_ave:', tmp_hottest_20yr_ave-tmp_coldest_20yr_ave

!...Checker (annual mortality for each size class) ...
!integer              ::size_class
!integer,dimension(10)::tree_number
!integer,dimension(10)::death_number
!integer              ::i

!size_class      = 0
!tree_number (:) = 0
!death_number(:) = 0

!Loop for each individual
do no=1, Max_no 
if ( tree_exist(no) ) then
if ( age(no)>1      ) then
   
   !set pft number
    p = pft(no)
   
   !mortality 1: growth efficiency
    greff      = (mort_regu1(no)/1000.0) / max(0.01, mort_regu2(no))
    greff      = max(0.0, greff)
    mort_greff = M1(p) / (M2(p)**greff)
    
    !other method to calculate mortality
    !greff      = mort_regu1(no) / max(0.1, mort_regu2(no)) / 1000.0
    !greff      = max(0.001, greff)
    !mort_greff = 1.0 / ( 1.0 + M1(p) * ( greff ** M2(p) ) )
    
   !mortality 2: heart stress
   if (Heat_sensitive(p)==1) then
      mort_heart = mort_regu3
   else
      mort_heart = 0.0
   end if
   
   !mortality 3: bioclimic limits (based on 20yrs mean of coldest month temperature)
   if     ( tmp_coldest_20yr_ave < TC_min(p) ) then
       mort_lim = 1.0
   elseif ( p==7 .and. (tmp_hottest_20yr_ave-tmp_coldest_20yr_ave)<43.0 ) then
       mort_lim = 1.0
   else
       mort_lim = 0.0
   endif
   
   !mortality 4: mortality by other factors
   if     ( height(no)-bole(no)<=2 .and. age(no)>3 ) then
      mort_etc = 1.0
   elseif ( mort_regu1(no)<10.0 .and. age(no)>3 )    then
      mort_etc = 1.0
   elseif ( dbh_heartwood(no)+dbh_sapwood(no) > 1.00 )   then
      mort_etc = 1.0
   elseif ( age(no) >= AGE_max(p) )                  then
      mort_etc = 1.0
   else
      mort_etc = 0.0
   endif
    
   !Sum up all mortality components, and calculate probabilty of death
   mort_total   = min(1.0, mort_greff + mort_heart + mort_lim + mort_etc)
   
   !...Checker...
   !size_class               = int( (dbh_heartwood(no)+dbh_sapwood(no)) / 0.05 ) + 1
   !size_class               = max(1, min(10, size_class))
   !tree_number (size_class) = tree_number(size_class)  + 1
   
   !When die
   if (mort_total > randf(I_seed)) then
   
   !...Checker...
   !death_number(size_class) = death_number(size_class) + 1
   Call die_tree &
    ( height(no), bole(no), dbh_heartwood(no), dbh_sapwood(no), &
    crown_diameter(no), crown_area(no), la(no), mort_regu2(no), &
    mass_leaf(no), mass_trunk(no), mass_root(no), mass_stock(no), mass_available(no), &
    age(no), pft(no), &
    height_limit(no), radius_limit(no), mort_regu1(no), &
    stat_leaf(no), stat_leaf_RunningRecord(no,:), &
    stat_leaf_crowntop(no), stat_leaf_crownbottom(no,:),&
    tree_exist(no), pool_litter )
   endif
   
end if
end if
end do

!increament of tree age
do no=1, Max_no 
if ( tree_exist(no) ) then !when tree exist
    age(no) = age(no) + 1  !add tree age
end if
end do

!...Checker...
!do i=1,10
!   tree_number(i) = max(1, tree_number(i))
!end do
!
!if (Logging==1) then
!   write(*,*) '---------- prop death ---------'
!   write(*,*) real(death_number(1)) / real(tree_number(1)), &
!              real(death_number(2)) / real(tree_number(2)), &
!              real(death_number(3)) / real(tree_number(3)), &
!              real(death_number(4)) / real(tree_number(4)), &
!              real(death_number(5)) / real(tree_number(5)), &
!              real(death_number(6)) / real(tree_number(6))
!   write(*,*) 
!endif

END SUBROUTINE mortality
