!**************************************************************************************************
! Relative intensity of diffused PAR for each forest layer
!**************************************************************************************************
SUBROUTINE diffused_radiation (par_diffuse_rel, pft, height, bole, la, tree_exist, phenology)

USE data_structure
implicit none

!Define augments
real,dimension(1:Max_hgt),intent(OUT)::par_diffuse_rel
                 !Relative intensity of diffused PAR for each forest layer (0.0-1.0)

integer,dimension(Max_no),intent(IN)::pft        !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no),intent(IN)::height     !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN)::bole       !bole length (STEP)
real   ,dimension(Max_no),intent(IN)::la         !leaf area (m2)
logical,dimension(Max_no),intent(IN)::tree_exist !tree exist or not (.TRUE. or .FALSE.)
logical,dimension(PFT_no),intent(IN)::phenology  !phenological status (T:foliation, F:dormance)

!Define local variables
real,dimension(PFT_no,Max_hgt)::lai  !LAI for each PFT for each forest layer (m2/m2)
real,dimension(1:Max_hgt)     ::attenuation_idx !attenuation index for each forest layer

integer no, i, p !loop counter
real    x        !for general usage

!Leaf Area Index for each layer for each PFT (lai)
   lai(:,:) = 0.0
   DO no = 1, Max_no
   if ( .not. tree_exist(no)     ) cycle
   if ( .not. phenology(pft(no)) ) cycle
      
      x = la(no) / (height(no)-bole(no)) !leaf area per each foliage step (m2/STEP)
      
      do i = bole(no)+1, height(no)
         lai(pft(no), i) = lai(pft(no), i) + x
      end do
   END DO
   lai(:,:) = lai(:,:) / real(Max_loc) / real(Max_loc)
   
!Light attenuation coefficent for each forest layer
   DO i = 1, Max_hgt
      x = 0.0
      do p = 1, PFT_no
         x = x + lai(p, i) * EK0(p)
      end do
      attenuation_idx(i) = x
   END DO
   
!Relative intensity of diffuse PAR for each forest layer (par_diffuse_rel)
   par_diffuse_rel(Max_hgt) = 1.0 !top layer
   DO i = Max_hgt, 1, -1
      par_diffuse_rel(i-1) = par_diffuse_rel(i) * exp( -1.0 * attenuation_idx(i) )
   END DO

END SUBROUTINE diffused_radiation



!**************************************************************************************************
! Direct PAR
! note: y_dimension -> East
!       x_dimension -> South
!**************************************************************************************************
SUBROUTINE direct_radiation &
    (par_direct_rel, sl_hgt, doy, LAT, phenology, &
     tree_exist, pft, height, bole, crown_diameter, crown_area, la, crown_x, crown_y)

USE data_structure
implicit none

!Target varialbe
real,dimension(Max_no,Max_hgt),intent(OUT)::par_direct_rel
          !relative intensity of direct PAR for each foliage layer (0.0-1.0)

integer                  ,intent(IN)::doy            !day of the year
real                     ,intent(IN)::LAT            !latitude for simulate
real                     ,intent(IN)::sl_hgt         !solar hight at midday (degree)
logical,dimension(PFT_no),intent(IN)::phenology      !Phenological status
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist flag
integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no),intent(IN)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN)::bole           !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_no),intent(IN)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no),intent(IN)::crown_area     !crown area     (m2)
real   ,dimension(Max_no),intent(IN)::la             !leaf area      (m2)
real   ,dimension(Max_no),intent(IN)::crown_x        !x_location in forest stand (m)
real   ,dimension(Max_no),intent(IN)::crown_y        !y_location in forest stand (m)

!Define augments
real                      ::sl_hgt_adj   !adjusted solar hight (degree)

real   ,dimension(Max_hgt)::overlap1 !(sum of) fraction overlap (self shading)
real   ,dimension(Max_hgt)::overlap2 !(sum of) fraction overlap (shading by proximate trees)
real   ,dimension(Max_no)::lad             !leaf area density for each crown layer (m2/m2/STEP)

integer,dimension(Max_no       )::save_value1
integer,dimension(Max_no*Max_no)::save_value2
integer,dimension(Max_no*Max_no)::save_value3
integer loop_no1 !for self-shading
integer loop_no2 !for neigbor-shading

real x3_me,  x4_me,  y1_me,  y2_me 
real x3_you, x4_you, y1_you, y2_you
real y_shift, xband_me, xband_you

integer no, me, you, i, j, loop             !for loop counter etc.
integer l1, l2                              !for temporal usage
real    x, y                                !for temporal usage
real    a1, a2, a3, b1, b2, c1, c2          !for temporal usage
real    dX                                  !
real    dist, dist_x, dist_y                !distance (m)
real    r, cosine, r1, r2, cosine1, cosine2 !for calculating circle overlapping
logical flag

!_____________ Set angle of virtual pipe
sl_hgt_adj = 0.86* sl_hgt

!_____________ Following procedures required only if sunlight exist
   par_direct_rel(:,:) = 0.00000 !(individual_no, layer_no)
   IF (sl_hgt_adj <= 10.0) return

!_____________ Select trees that requires computation
   loop_no1 = 0
   DO no = 1, Max_no
   if ( .not. tree_exist(no)     ) cycle
   if ( .not. phenology(pft(no)) ) cycle
      !tree number that requires computation
      loop_no1              = loop_no1 + 1  
      !ID number that requires computation  
      save_value1(loop_no1) = no            
      !Leaf Area Density for each tree (m2/m2/STEP)
      lad(no) = la(no) / max(0.001,crown_area(no)) / real( max(1,height(no)-bole(no)) )
   END DO

!_____________ Following procedures required only if foliation tree exist
   IF (loop_no1 == 0) return

!_____________ Initialize
   !give default values
   par_direct_rel(:,:) = 1.00000 !(individual_no, layer_no)
   
   !dX: shade lengh for single height step (unit=m/STEP)
   dX = STEP / tan(sl_hgt_adj*DtoR)

!_____________ Self shading
DO loop = 1, loop_no1             !For each tree
   me = save_value1(loop)         !ID number of the current tree
   r  = crown_diameter(me) / 2.0  !crown radius (m)
   if ( height(me)-bole(me) < 2 ) cycle
   
   !compute fraction of over lap between disk and shade
   overlap1(:) = 0.0
   do j = 1, ( height(me)-bole(me)-1 ) !For each distance class between disks
      !dist: x-axis distance between shade and current disk
      dist = mod( dX*real(j) , real(Max_loc) )
      
      !obtain fraction of area overlap1
      IF     ( dist<=0.0 )       then
      overlap1(j) = 1.0
      
      ELSEIF ( 2.0 * r <= dist ) then
      overlap1(j) = 0.0
      
      ELSEIF ( r < dist )        then
      cosine = min(1.000, max(0.000, (dist**2) / (2.0*r*dist) ))
      overlap1(j) = (2/PI)*( acos(cosine) - cosine * sqrt(1.0 - cosine**2) )
      
      ELSE
      cosine = min(1.000, max(0.000, (dist**2) / (2.0*r*dist) ))
      overlap1(j) = 2.0*acos(cosine)/PI - (dist/(r*r*PI)) * sqrt( r*r - (r*cosine-dist)**2 )
      
      ENDIF
   enddo
   
   !sumup fraction disk overlap1
   do j = 2, max(2, height(me)-bole(me)-1)
      overlap1(j) = overlap1(j) + overlap1(j-1)
   end do
   
   !set par_direct_rel for each crown disk
   x = -1.000 * EK0(pft(me)) * lad(me)
   do i = bole(me)+1, height(me)-1 !i: location of shade-receiving disk
      j = height(me) - i           !j: number of disk over the current disk
      par_direct_rel(me,i) = exp(x*overlap1(j))
   enddo
   
END DO

!_____________ Following procedures required only if foliation tree exist more than 2
IF (loop_no1 < 2) return

!_____________ List up tree pairs for computation
loop_no2 = 0
do i = 1, loop_no1 !tree 'me' 
do j = 1, loop_no1 !tree 'you'
   
   !recall tree number
   me  = save_value1(i)
   you = save_value1(j)
   
   !cancel apparently no-interference trees
   if ( bole(me)+1 >= height(you) ) cycle
   if ( i == j )                    cycle
   
   !cancel no-interferebce trees
   y1_me  = crown_y(me)  + crown_diameter(me)  / 2.0
   y2_me  = crown_y(me)  - crown_diameter(me)  / 2.0
   y1_you = crown_y(you) + crown_diameter(you) / 2.0
   y2_you = crown_y(you) - crown_diameter(you) / 2.0
   
   a1 = abs( y2_me - (y2_you        ) )
   a2 = abs( y2_me - (y2_you+Max_loc) )
   a3 = abs( y2_me - (y2_you-Max_loc) )
   
   y_shift = 0.0
   if ( a2<=a1 .and. a2<=a3 ) y_shift =        real(Max_loc)
   if ( a3<=a1 .and. a3<=a2 ) y_shift = -1.0 * real(Max_loc)
   
   b1 = min(y1_me, y1_you + y_shift)
   b2 = max(y2_me, y2_you + y_shift)
   if ( b1 < b2 ) cycle
   
   if ( (crown_y(me)<b1) .and. (crown_y(me)>b2) ) then
      xband_me = crown_diameter(me)
   else
      x = 0.25 * ( crown_diameter(me)**2 )
      y = min( abs(crown_y(me)-b1) , abs(crown_y(me)-b2) ); y = y**2
      xband_me = 2.0 * sqrt(abs(x-y))
   endif
   
   if ( (crown_y(you)+y_shift<b1) .and. (crown_y(you)+y_shift>b2) ) then
      xband_you = crown_diameter(you)
   else
      x = 0.25 * ( crown_diameter(you)**2 )
      y = min( abs(crown_y(you)+y_shift-b1) , abs(crown_y(you)+y_shift-b2) ); y = y**2
      xband_you = 2.0 * sqrt(abs(x-y))
   endif
   
   x3_me  = crown_x(me)  + xband_me  / 2.0
   x4_me  = crown_x(me)  - xband_me  / 2.0
   x3_you = crown_x(you) + xband_you / 2.0
   x4_you = crown_x(you) - xband_you / 2.0
   
   flag = .true.
   if (sl_hgt_adj<90.0) then
      c1 = x3_me + dX * (height(you)-bole(me)-1)
      c2 = x4_me + dX * real( max( height(me) , bole(you)+1 ) - height(me) )
      if ( c1>x4_you         .and. c2<x3_you         ) flag= .false.
      if ( c1>x4_you+Max_loc .and. c2<x3_you+Max_loc ) flag= .false.
   else
     c1 = x4_me - dX * real(height(you)-bole(me)-1)
     c2 = x3_me - dX * real( max( height(me) , bole(you)+1 ) - height(me) )
     if ( c1<x3_you         .and. c2>x4_you         ) flag= .false.
     if ( c1<x3_you-Max_loc .and. c2>x4_you-Max_loc ) flag= .false.
   endif
   if (flag) cycle
   
   !save values
   loop_no2              = loop_no2 + 1
   save_value2(loop_no2) = me          
   save_value3(loop_no2) = you         
   
end do
end do

!_____________ Among trees shading
!CDIR LOOPCNT = 15000
DO loop = 1, loop_no2 !for each combination of individuals
   
   overlap2(:) = 0.0
   
   !recall tree number for comparison
   me     = save_value2(loop) !ID of current tree
   you    = save_value3(loop) !ID of target tree
   
   !y: y-axis distance between crown centers
   dist_y = abs(crown_y(me)-crown_y(you)) !absolute Y-distance between crown center (0..Max_loc)
   dist_y = min(y, Max_loc - y)           !adjustment for repeat world
   
   !fraction of overlapping shade area
   do j = max( 1 , bole(you)+1-height(me) ), height(you)-bole(me)-1
      !dist_x: x-axis distance between crown and shade center
      if (sl_hgt_adj<=90.0) then
         dist_x = abs( crown_x(me) - (crown_x(you)-dX*real(j)) )
         dist_x = min( dist_x , Max_loc - dist_x )
      else
         dist_x = abs( crown_x(me) - (crown_x(you)+dX*real(j)) )
         dist_x = min( dist_x , Max_loc - dist_x )
      endif
      
      !dist: x-y-plane distance between crown and shade center
      dist = sqrt(dist_x**2 + dist_y**2)
      
      !calculate overlapping area
      r1 = min(crown_diameter(me),crown_diameter(you)) / 2.0 !radius of smaller circle (m)
      r2 = max(crown_diameter(me),crown_diameter(you)) / 2.0 !radius of larger  circle (m)
      if     (r1+r2 <= dist) then
         y = 0.000
         
      elseif (dist <= r2-r1) then
         y = PI * r1 * r1
         y = min(1.000, y / crown_area(me))
         
      elseif (r2 < dist ) then
         cosine1 = min(1.000, max(0.000, (r1**2 - r2**2 + dist**2) / (2.0*r1*dist) ))
         cosine2 = min(1.000, max(0.000, (r2**2 - r1**2 + dist**2) / (2.0*r2*dist) ))
         y       = r1*r1*( acos(cosine1) - cosine1 * sqrt(1.0 - cosine1**2) ) &
                 + r2*r2*( acos(cosine2) - cosine2 * sqrt(1.0 - cosine2**2) )  
         y       = min(1.000, y / crown_area(me))
         
      else
         cosine1 = min(1.000, max(0.000, (r1**2 - r2**2 + dist**2) / (2.0*r1*dist) ))
         cosine2 = min(1.000, max(0.000, (r2**2 - r1**2 + dist**2) / (2.0*r2*dist) ))
         y       = r1*r1*acos(cosine1) + r2*r2*acos(cosine2)   &
                 - dist * sqrt( r1*r1 - (r2*cosine2-dist)**2 )  
         y       = min(1.000, y / crown_area(me))
         
      end if
      overlap2(j) = y
   end do
   
   !reflect shadeing to par_direct_rel for each crown disk
   do i = bole(me)+1, min(height(me), height(you)-1)
      l1 = max(1,   bole(you) + 1 - i) !lower
      l2 = max(1, height(you)     - i) !upper
      x  = sum(overlap2(l1:l2))
      par_direct_rel(me,i) = par_direct_rel(me,i) * exp(-1.000 * EK0(pft(you)) * lad(you) * x)
   end do
   
END DO

END SUBROUTINE direct_radiation



!**************************************************************************************************
! obtain ground vacant where new sapling can establish
!**************************************************************************************************
SUBROUTINE ground_vacant &
         (patch_vacant, bole, crown_diameter, bole_x, bole_y, crown_x, crown_y, tree_exist)

USE data_structure
implicit none

!Define parameters
real   ,parameter::Crown_diameter_new = 1.0 !maximum crown diameter of new tree (m)
integer,parameter::Height_new         = 20  !maximum height of new tree      (step)

!Define arguments
logical,dimension(1:Dived,1:Dived),intent(OUT)::patch_vacant !patch flag for tree establishment

integer,dimension(1:Max_no),intent(IN)::bole           !bole length (STEP)
real   ,dimension(1:Max_no),intent(IN)::crown_diameter !crown diameter      (m)
real   ,dimension(1:Max_no),intent(IN)::bole_x         !x_location of bole  (m)
real   ,dimension(1:Max_no),intent(IN)::bole_y         !y_location of bole  (m)
real   ,dimension(1:Max_no),intent(IN)::crown_x        !x_location of crown (m)
real   ,dimension(1:Max_no),intent(IN)::crown_y        !y_location of crown (m)
logical,dimension(1:Max_no),intent(IN)::tree_exist     !tree exist or not (.TRUE. or .FALSE.)

!Define local variables
real    distance                           !distance (m)
real    d1, d2, d3, d4, d5, d6, d7, d8, d9 !distance (m)
real    x, y                               !for temporal usage
integer no, i, j                           !for loop counter

!Reset output variables
patch_vacant(:,:) = .true.

!Calculate safe-site in relation to proximity of previous trees
Do no=1, Max_no
if ( .not. tree_exist(no)   ) cycle
   
   !two trees cannot exist at the same establishment location
   x = real(Dived)/real(Max_loc)
   patch_vacant( int(bole_x(no)*x)+1 , int(bole_y(no)*x)+1 ) = .false.
   
if ( bole(no) >= Height_new ) cycle
   
   !examine for proximate area for each grid of establishment coodinate
   do i = 1, Dived
   do j = 1, Dived
      
      !location of this tree under nomal coodinate
      x = real(i) * real(Max_loc) / real(Dived) !x location (m)
      y = real(j) * real(Max_loc) / real(Dived) !y location (m)
      
      !calculate distance between my place and target place
      d1 = (crown_x(no)-Max_loc-x)**2 + (crown_y(no)-Max_loc-y)**2
      d2 = (crown_x(no)        -x)**2 + (crown_y(no)-Max_loc-y)**2
      d3 = (crown_x(no)+Max_loc-x)**2 + (crown_y(no)-Max_loc-y)**2
      d4 = (crown_x(no)-Max_loc-x)**2 + (crown_y(no)        -y)**2
      d5 = (crown_x(no)        -x)**2 + (crown_y(no)        -y)**2
      d6 = (crown_x(no)+Max_loc-x)**2 + (crown_y(no)        -y)**2
      d7 = (crown_x(no)-Max_loc-x)**2 + (crown_y(no)+Max_loc-y)**2
      d8 = (crown_x(no)        -x)**2 + (crown_y(no)+Max_loc-y)**2
      d9 = (crown_x(no)+Max_loc-x)**2 + (crown_y(no)+Max_loc-y)**2
      
      distance = min(d1,d2,d3,d4,d5,d6,d7,d8,d9)
      distance = sqrt(distance)
      
      !determine whether current site is safe or not
      if ( distance < 0.5 * (crown_diameter(no)+Crown_diameter_new) ) then
         patch_vacant(i,j) = .false.
      endif
   end do
   end do
   
End do

END SUBROUTINE ground_vacant



!**************************************************************************************************
! Calculate spacial limitation on growth (for woody PFTs, Monthely computation)
!**************************************************************************************************
SUBROUTINE spacial_limitation &
      (height_limit, radius_limit, &
      pft, height, bole, crown_diameter, crown_x, crown_y, tree_exist, phenology)

USE data_structure
implicit none

!Define parameters
real,parameter::Margine=0.1  !minimum proximate distance between crowns (m)

!Define arguments
integer,dimension(1:Max_no),intent(OUT)::height_limit
                                         !maximum tree height that proximate trees permit (step)
real   ,dimension(1:Max_no),intent(OUT)::radius_limit
                                         !maximum canopy radius that proximate trees permit (m)
integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no),intent(IN)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN)::bole           !bole length (STEP)
real   ,dimension(Max_no),intent(IN)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no),intent(IN)::crown_x        !x_location of crown (m)
real   ,dimension(Max_no),intent(IN)::crown_y        !y_location of crown (m)
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist or not (.TRUE. or .FALSE.)
logical,dimension(PFT_no),intent(IN)::phenology      !leaf status (T:foliation, F:dormance)

!Define local variable
real    proxy, x, y
real    d1, d2, d3, d4, d5, d6, d7, d8, d9
integer me, you

!** for each combination of trees
DO me =1, Max_no
if ( .not. tree_exist(me)     ) cycle
if ( .not. phenology(pft(me)) ) cycle
   
   !give potential maximum
   height_limit(me) = int( (HGT_max(pft(me))-1.3) / STEP )
   radius_limit(me) = CD_max(pft(me)) / 2.0                  
   
   !examine limitations of proxymate trees
   DO you=1, Max_no
   if ( .not. tree_exist(you)     ) cycle !case tree not exist
   if ( bole  (me) >= height(you) ) cycle !case no interference
   if ( me         == you         ) cycle !case same tree
      
      !relative location of each 9 duplicated-target-tree
      x = crown_x(you) - crown_x(me) !difference in x-location
      y = crown_y(you) - crown_y(me) !difference in y-location
      
      d1 = (x - real(Max_loc))**2 + (y - real(Max_loc))**2
      d2 = (x                )**2 + (y - real(Max_loc))**2
      d3 = (x + real(Max_loc))**2 + (y - real(Max_loc))**2
      d4 = (x - real(Max_loc))**2 + (y                )**2
      d5 = (x                )**2 + (y                )**2
      d6 = (x + real(Max_loc))**2 + (y                )**2
      d7 = (x - real(Max_loc))**2 + (y + real(Max_loc))**2
      d8 = (x                )**2 + (y + real(Max_loc))**2
      d9 = (x + real(Max_loc))**2 + (y + real(Max_loc))**2
      
      !obtain most proximate distance among 9 duplicated-target-trees
      proxy = min(d1,d2,d3,d4,d5,d6,d7,d8,d9)
      proxy = sqrt(proxy)
      
      !Height growth limit
      if ( proxy < crown_diameter(me)/2 + crown_diameter(you)/2 + Margine ) then
      if ( bole(you) >= height(me) ) then
         height_limit(me) = min( height_limit(me), bole(you) )
      end if
      end if
      
      !Radium growth limit
      if ( bole(me) < height(you) .and. height(me) > bole(you) ) then
         x = proxy - crown_diameter(you)/2 - Margine
         radius_limit(me) = min( radius_limit(me), x )
      end if
      
   ENDDO
   
ENDDO

END SUBROUTINE spacial_limitation



!**************************************************************************************************
! Crown movement
!**************************************************************************************************
SUBROUTINE crown_shake (crown_x, crown_y, &
                        bole_x, bole_y, bole, height, pft, crown_diameter, tree_exist)

USE data_structure
implicit none

!Define parameters
integer,parameter::Direct_resolusion = 36  !resolution of movement direction   (n)
real   ,parameter::Crown_plasticity  = 0.5 !maximum fraction of crown movement (fraction)
real   ,parameter::Crown_max_move    = 0.2 !maximum movement of crown center   (m/year)
real   ,parameter::Search_radius     = 3.0 ! (m)
integer,parameter::Max_proxy         = 100 !maximum number of proximate trees to examine (n)

!Define arguments
real   ,dimension(Max_no),intent(INOUT)::crown_x      !x_location of crown (m)
real   ,dimension(Max_no),intent(INOUT)::crown_y      !y_location of crown (m)
real   ,dimension(Max_no),intent(IN) ::bole_x         !x_location of crown (m)
real   ,dimension(Max_no),intent(IN) ::bole_y         !y_location of crown (m)
integer,dimension(Max_no),intent(IN) ::bole           !bole length (STEP)
integer,dimension(Max_no),intent(IN) ::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN) ::pft            !Plant Functional Type (1-PFT_no)
real   ,dimension(Max_no),intent(IN) ::crown_diameter !crown diameter (m)
logical,dimension(Max_no),intent(IN) ::tree_exist     !tree exist or not (.TRUE. or .FALSE.)

!Define local variable
real,dimension(Direct_resolusion)::lenght_max1 !lengh max of crown movement (m)
real,dimension(Direct_resolusion)::lenght_max2 !lengh max of crown movement (m)
real,dimension(Max_proxy, 3)     ::proxy_crown !(no_you, your r/crown_x/crown_y)
real    angle, angle0, angle1, angle2 !angle (radius)
real    x, y, z                       !for temporal usage
real    dx, dy, r, d                  !for temporal usage
real    a1, a2, a3                    !for temporal usage
real    distance                      !
integer direction                     !
integer me, you, counter, i, j        !for temporal counter


!----------------   For each existing tree   ----------------
DO me=1, Max_no
IF ( .not. tree_exist(me) )  cycle
   
!** Determine maximum plasticity of crown movement for each direction
!   output --> lenght_max1 (Direct_resolusion)
   
   !obtain geometric values
   dx = crown_x(me) - bole_x(me) !dx between bole_center and crown_center
   dy = crown_y(me) - bole_y(me) !dy between bole_center and crown_center
   
   r = 0.5 * crown_diameter(me) * Crown_plasticity !r
   d = sqrt( dx**2 + dy**2 )                       !d
   
IF (d==0.0) then !when bole_center == crown_center
   
   lenght_max1(:) = r
   
ELSE
   angle0 = acos( -1.0 * dy / d )
   
   !for each direction of the movement
   DO i=1, Direct_resolusion
      !obtain angle2
      angle1 = 2.0 * PI * ( real(i) / real(Direct_resolusion) )
      
      if ( dx >= 0.0 ) then
         angle2 = angle0 + angle1
      else
         angle2 = abs(angle0 - angle1)
      endif
      
      do while (angle2 > 2.0*PI); angle2 = angle2 - 2.0*PI; enddo
      
      if ( angle2 > PI ) angle2 = 2.0*PI - angle2
      
      !give maximum movement
      z = d*d*cos(angle2)*cos(angle2) + r*r - d*d
      if (z < 0.0) then
         lenght_max1(i) = 0.0
      else
         lenght_max1(i) = d*cos(angle2) + sqrt(z)
      endif
      
   END DO
   
ENDIF
   
!** Find crowns that may cause interference
!   output --> proxy_crown (Max_proxy, 3)
   
   !initialize
   counter = 0
   proxy_crown(:,:) = 0.0
   
   !for each tree
   DO you = 1, Max_no
   if ( counter==Max_proxy        ) exit                       
   if ( .not. tree_exist(you)     ) cycle !case tree not exist 
   if ( bole  (me) >  height(you) ) cycle !case no interference
   if ( height(me) <= bole  (you) ) cycle !case no interference
   if ( me         == you         ) cycle !case same tree      
      
      !common variables in the following loop
      a1 = crown_x(you) - crown_x(me)
      a2 = crown_y(you) - crown_y(me)
      a3 = (crown_diameter(me)+crown_diameter(you))/2.0 + max(Search_radius, Crown_max_move)
      
      DO i=1, 9
         
         !x, y: rekative location of the target crown
         select case (i)
           case(1); dx = a1 - Max_loc ; dy = a2 - Max_loc
           case(2); dx = a1           ; dy = a2 - Max_loc
           case(3); dx = a1 + Max_loc ; dy = a2 - Max_loc
           case(4); dx = a1 - Max_loc ; dy = a2          
           case(5); dx = a1           ; dy = a2          
           case(6); dx = a1 + Max_loc ; dy = a2          
           case(7); dx = a1 - Max_loc ; dy = a2 + Max_loc
           case(8); dx = a1           ; dy = a2 + Max_loc
           case(9); dx = a1 + Max_loc ; dy = a2 + Max_loc
         end select
         
         !list up for all interference crown
         if ( counter==Max_proxy      ) exit
         if ( sqrt(dx*dx+dy*dy) >= a3 ) cycle
         if ( dx==0.0 .and. dy==0.0   ) cycle
         
         counter = counter + 1
         proxy_crown(counter, 1) = 0.5*crown_diameter(you) !radius (m)
         proxy_crown(counter, 2) = dx                      !relative x location (m)
         proxy_crown(counter, 3) = dy                      !relative y location (m)
         
      ENDDO
   ENDDO
   
!** Adjust maximum movement of crown center by proxymate trees
!   output --> lenght_max2 (Direct_resolusion)
   
   !initialize
   lenght_max2(:) = Search_radius
   
   !for each proxymate tree
   DO counter=1, Max_proxy
      
      r        = proxy_crown(counter, 1) !radius (m)
      dx       = proxy_crown(counter, 2) !relative x location (m)
      dy       = proxy_crown(counter, 3) !relative y location (m)
      distance = sqrt( dx*dx + dy*dy )
      
      if ( r==0.0 )  exit !case no interference crown remains
      
      if ( dx >= 0.0 ) then
         angle0 =          acos( dy / distance )
      else
         angle0 = 2.0*PI - acos( dy / distance )
      endif
      
      !for each direction
      DO i=1, Direct_resolusion
         angle1 = 2.0 * PI * ( real(i) / real(Direct_resolusion) )
         angle2 = angle0 - angle1
         
         !covert target crown location on the new coodinate system
         x = distance * sin(angle2) !converted x (m)
         y = distance * cos(angle2) !converted y (m)
         
         !select interference situation
         if ( y <= 0.0                       ) cycle
         if (  0.5*crown_diameter(me) <= x-r ) cycle
         if ( -0.5*crown_diameter(me) >= x+r ) cycle
         
         !adjust maximum movement, z (m)
         z = y - sqrt( (0.5*crown_diameter(me)+r)**2 - x**2 )
         z = max(0.0, z)
         lenght_max2(i) = min( lenght_max2(i), z )
         
      ENDDO
   ENDDO
   
!** Execute crown movement
!   output --> crown_x, crown_y
IF ( minval(lenght_max2(:)) .ne. maxval(lenght_max2(:)) ) then
   
   !determine direction and distance of crown movement
   direction = 1
   distance  = 0.0
   DO i=1, Direct_resolusion  !for each direction
      j = mod(i+me, Direct_resolusion) + 1
      
      if ( lenght_max2(j) > distance ) then
         direction = j
         distance  = lenght_max2(j)
      endif
   ENDDO
   angle = 2.0 * PI * ( real(direction) / real(Direct_resolusion) )
   distance  = min( lenght_max2(direction), lenght_max1(direction), Crown_max_move )
   
   !execute crown movement
   crown_x(me) = crown_x(me) + distance * sin(angle)
   crown_y(me) = crown_y(me) + distance * cos(angle)
   
ENDIF

ENDDO
!------------------------------------------------------------

END SUBROUTINE crown_shake
